@@ -17,11 +17,13 @@
 
 import concurrent.futures
 import difflib
+import os
 from typing import Any, Dict, FrozenSet, Iterator, List, Type
 
 from absl import flags
 from absl import logging
 
+from etils import epath
 import tensorflow as tf
 from tensorflow_datasets.core import dataset_builder
 from tensorflow_datasets.core import naming
@@ -29,32 +31,21 @@ from tensorflow_datasets.core import read_only_builder
 from tensorflow_datasets.core import registered
 from tensorflow_datasets.core import utils
 from tensorflow_datasets.core.community import register_base
-import toml
 
 TFDS_DEBUG_VERBOSE = flags.DEFINE_boolean('tfds_debug_list_dir', False,
                                           'Debug the catalog generation')
 
-ListOrElem = utils.ListOrElem
-
-# pylint: disable=logging-format-interpolation
+# pylint: disable=logging-fstring-interpolation
 
 
 class DataDirRegister(register_base.BaseRegister):
   """Dataset register based on generated `data_dir` paths.
 
-  This register map `namespace` strings to `data_dir` paths. Mapping is defined
-  in `.toml` format:
-
-  ```toml
-  [Namespaces]
-  kaggle='/path/to/datasets/'
-  tensorflow_graphics='gs://tensorflow-graphics/datasets'
-  ```
-
   Usage:
 
   ```python
-  register = DataDirRegister(path='/path/to/namespaces.toml')
+  register = DataDirRegister(namespace_to_data_dirs=
+    {'my_namespace': [epath.Path('/path/to/namespaces.toml')]})
 
   # List all registered datasets: ['kaggle:ds0', 'kaggle:ds1',...]
   register.list_builders()
@@ -65,24 +56,14 @@ class DataDirRegister(register_base.BaseRegister):
 
   """
 
-  def __init__(self, path: utils.PathLike):
+  def __init__(self, namespace_to_data_dirs: Dict[str, List[epath.Path]]):
     """Contructor.
 
     Args:
-      path: Path to the register files containing the mapping namespace ->
-        data_dir
+      namespace_to_data_dirs: Mapping from namespace to list of paths where the
+        datasets for that namespace are located.
     """
-    self._path: utils.ReadOnlyPath = utils.as_path(path)
-
-  @utils.memoized_property
-  def _ns2data_dir(self) -> Dict[str, List[utils.ReadWritePath]]:
-    """Mapping `namespace` -> `data_dir`."""
-    # Lazy-load the namespaces the first requested time.
-    config = toml.loads(self._path.read_text())
-    return {
-        namespace: _as_path_list(path_or_paths)
-        for namespace, path_or_paths in config['Namespaces'].items()
-    }
+    self._ns2data_dir = namespace_to_data_dirs
 
   @utils.memoized_property
   def namespaces(self) -> FrozenSet[str]:
@@ -127,24 +108,18 @@ class DataDirRegister(register_base.BaseRegister):
                      f'Should be one of {sorted(self._ns2data_dir)}{hint}')
     return read_only_builder.builder_from_files(
         name.name,
-        data_dir=self._ns2data_dir[name.namespace],
+        data_dir=[
+            os.fspath(path) for path in self._ns2data_dir[name.namespace]
+        ],
         **builder_kwargs,
     )
 
-  def get_builder_root_dirs(
-      self, name: utils.DatasetName) -> List[utils.ReadWritePath]:
+  def get_builder_root_dirs(self, name: utils.DatasetName) -> List[epath.Path]:
     """Returns root dir of the generated builder (without version/config)."""
     return [d / name.name for d in self._ns2data_dir[name.namespace]]
 
 
-def _as_path_list(path_or_paths: ListOrElem[str]) -> List[utils.ReadWritePath]:
-  if isinstance(path_or_paths, list):
-    return [utils.as_path(p) for p in path_or_paths]
-  else:
-    return [utils.as_path(path_or_paths)]
-
-
-def _maybe_iterdir(path: utils.ReadOnlyPath) -> Iterator[utils.ReadOnlyPath]:
+def _maybe_iterdir(path: epath.Path) -> Iterator[epath.Path]:
   """Same as `path.iterdir()`, but don't fail if path does not exist."""
   # Use try/except rather than `.exists()` to avoid an extra RPC call
   # per namespace
@@ -152,7 +127,9 @@ def _maybe_iterdir(path: utils.ReadOnlyPath) -> Iterator[utils.ReadOnlyPath]:
     for f in path.iterdir():
       yield f
   except (
+      OSError,
       FileNotFoundError,
+      PermissionError,
       tf.errors.NotFoundError,
       tf.errors.PermissionDeniedError,
   ) as e:
@@ -160,7 +137,7 @@ def _maybe_iterdir(path: utils.ReadOnlyPath) -> Iterator[utils.ReadOnlyPath]:
 
 
 def _iter_builder_names(
-    ns2data_dir: Dict[str, List[utils.ReadOnlyPath]],) -> Iterator[str]:
+    ns2data_dir: Dict[str, List[epath.Path]],) -> Iterator[str]:
   """Yields the `ns:name` dataset names."""
   FILTERED_DIRNAME = frozenset(('downloads',))  # pylint: disable=invalid-name
 
@@ -171,7 +148,7 @@ def _iter_builder_names(
   # For better performances, load all namespaces asynchonously
   def _get_builder_names_single_namespace(
       ns_name: str,
-      data_dir: utils.ReadOnlyPath,
+      data_dir: epath.Path,
   ) -> List[str]:
     # Note: `data_dir` might contain non-dataset folders, but checking
     # individual dataset would have significant performance drop, so
