@@ -27,7 +27,9 @@ from tensorflow_datasets import testing
 from tensorflow_datasets.core import dataset_info
 from tensorflow_datasets.core import features
 from tensorflow_datasets.core import file_adapters
+from tensorflow_datasets.core import naming
 from tensorflow_datasets.core import read_only_builder
+from tensorflow_datasets.core import splits as splits_lib
 from tensorflow_datasets.core import utils
 from tensorflow_datasets.core.proto import dataset_info_pb2
 from tensorflow_datasets.core.proto import feature_pb2
@@ -145,7 +147,7 @@ class DatasetInfoTest(testing.TestCase):
     info.read_from_directory(_INFO_DIR)
 
     # Read the json file into a string.
-    with tf.io.gfile.GFile(info._dataset_info_path(_INFO_DIR)) as f:
+    with tf.io.gfile.GFile(dataset_info.dataset_info_path(_INFO_DIR)) as f:
       existing_json = json.load(f)
 
     # Now write to a temp directory.
@@ -153,11 +155,11 @@ class DatasetInfoTest(testing.TestCase):
       info.write_to_directory(tmp_dir)
 
       # Read the newly written json file into a string.
-      with tf.io.gfile.GFile(info._dataset_info_path(tmp_dir)) as f:
+      with tf.io.gfile.GFile(dataset_info.dataset_info_path(tmp_dir)) as f:
         new_json = json.load(f)
 
       # Read the newly written LICENSE file into a string.
-      with tf.io.gfile.GFile(info._license_path(tmp_dir)) as f:
+      with tf.io.gfile.GFile(dataset_info.license_path(tmp_dir)) as f:
         license_ = f.read()
 
     # Assert what was read and then written and read again is the same.
@@ -181,8 +183,11 @@ class DatasetInfoTest(testing.TestCase):
         license="some license",
     )
     info.download_size = 456
-    info.as_proto.splits.add(name="train", num_bytes=512)
-    info.as_proto.splits.add(name="validation", num_bytes=64)
+    filepath_template = "{DATASET}-{SPLIT}.{FILEFORMAT}-{SHARD_X_OF_Y}"
+    info.as_proto.splits.add(
+        name="train", num_bytes=512, filepath_template=filepath_template)
+    info.as_proto.splits.add(
+        name="validation", num_bytes=64, filepath_template=filepath_template)
     info.as_proto.schema.feature.add()
     info.as_proto.schema.feature.add()  # Add dynamic statistics
     info.download_checksums = {
@@ -300,6 +305,72 @@ class DatasetInfoTest(testing.TestCase):
     self.assertEqual(40, info.splits.total_num_examples)
     self.assertEqual(2, len(info.as_proto.schema.feature))
 
+  def test_set_splits_normal(self):
+    info = dataset_info.DatasetInfo(builder=self._builder)
+    split_info1 = splits_lib.SplitInfo(
+        name="train", shard_lengths=[1, 2], num_bytes=0)
+    split_info2 = splits_lib.SplitInfo(
+        name="test", shard_lengths=[1], num_bytes=0)
+    split_dict = splits_lib.SplitDict(split_infos=[split_info1, split_info2])
+    info.set_splits(split_dict)
+    self.assertEqual(str(info.splits), str(split_dict))
+    self.assertEqual(
+        str(info.as_proto.splits),
+        str([split_info1.to_proto(),
+             split_info2.to_proto()]))
+
+  def test_set_splits_incorrect_dataset_name(self):
+    info = dataset_info.DatasetInfo(builder=self._builder)
+    split_info1 = splits_lib.SplitInfo(
+        name="train",
+        shard_lengths=[1, 2],
+        num_bytes=0,
+        filename_template=naming.ShardedFileTemplate(
+            dataset_name="some_other_dataset",
+            split="train",
+            data_dir=info.data_dir,
+            filetype_suffix="tfrecord"))
+    split_dict = splits_lib.SplitDict(split_infos=[split_info1])
+    with pytest.raises(
+        AssertionError, match="SplitDict contains SplitInfo for split"):
+      info.set_splits(split_dict)
+
+  def test_set_splits_multi_split_info(self):
+    info = dataset_info.DatasetInfo(builder=self._builder)
+    split_info1 = splits_lib.SplitInfo(
+        name="train", shard_lengths=[1, 2], num_bytes=0)
+    split_info2 = splits_lib.SplitInfo(
+        name="test", shard_lengths=[1], num_bytes=0)
+    multi_split_info1 = splits_lib.MultiSplitInfo(
+        name="train", split_infos=[split_info1])
+    multi_split_info2 = splits_lib.MultiSplitInfo(
+        name="test", split_infos=[split_info2])
+    split_dict = splits_lib.SplitDict(
+        split_infos=[multi_split_info1, multi_split_info2])
+    info.set_splits(split_dict)
+    self.assertEqual(str(info.splits), str(split_dict))
+    self.assertEqual(
+        str(info.as_proto.splits),
+        str([split_info1.to_proto(),
+             split_info2.to_proto()]))
+
+  def test_set_file_format_override_fails(self):
+    info = dataset_info.DatasetInfo(builder=self._builder)
+    info.set_file_format(file_adapters.FileFormat.TFRECORD)
+    self.assertEqual(info.file_format, file_adapters.FileFormat.TFRECORD)
+    with pytest.raises(
+        ValueError,
+        match="File format is already set to FileFormat.TFRECORD. Got FileFormat.RIEGELI"
+    ):
+      info.set_file_format(file_adapters.FileFormat.RIEGELI)
+
+  def test_set_file_format_override(self):
+    info = dataset_info.DatasetInfo(builder=self._builder)
+    info.set_file_format(file_adapters.FileFormat.TFRECORD)
+    self.assertEqual(info.file_format, file_adapters.FileFormat.TFRECORD)
+    info.set_file_format(file_adapters.FileFormat.RIEGELI, override=True)
+    self.assertEqual(info.file_format, file_adapters.FileFormat.RIEGELI)
+
 
 @pytest.mark.parametrize(
     "file_format",
@@ -475,6 +546,7 @@ _INFO_STR = '''tfds.core.DatasetInfo(
     """,
     homepage='https://storage.googleapis.com/cvdf-datasets/mnist/',
     data_path='%s',
+    file_format=tfrecord,
     download_size=1.95 KiB,
     dataset_size=11.06 MiB,
     features=FeaturesDict({
