@@ -21,8 +21,6 @@ from tensorflow_datasets.core.units import Size
 from tensorflow_datasets.core.utils import docs
 from tensorflow_datasets.core.utils import tree_utils as tree
 from tensorflow_datasets.core.utils.gcs_utils import gcs_path
-from tensorflow_datasets.core.utils.generic_path import as_path
-from tensorflow_datasets.core.utils.generic_path import register_pathlike_cls
 from tensorflow_datasets.core.utils.image_utils import *
 from tensorflow_datasets.core.utils.py_utils import *
 from tensorflow_datasets.core.utils.resource_utils import *
