@@ -15,11 +15,11 @@
 
 """Benchmark utils."""
 
+import dataclasses
 import time
 from typing import Any, Dict, Iterable, Optional, Union
 
 from absl import logging
-import dataclasses
 from tensorflow_datasets.core.utils import tqdm_utils
 
 try:
@@ -53,7 +53,8 @@ def benchmark(
   Usage:
 
   ```py
-  ds = tfds.load('mnist', split='train').batch(32).prefetch()
+  ds = tfds.load('mnist', split='train')
+  ds = ds.batch(32).prefetch(buffer_size=tf.data.AUTOTUNE)
   tfds.benchmark(ds, batch_size=32)
   ```
 
