@@ -16,13 +16,17 @@
 """Test utilities."""
 
 import contextlib
-import io
+import dataclasses
+import functools
 import os
+import pathlib
 import subprocess
 import tempfile
 from typing import Any, Iterator
 from unittest import mock
 
+from etils import epath
+from etils import epy
 import numpy as np
 import tensorflow as tf
 
@@ -64,35 +68,46 @@ def fake_examples_dir():
   return os.path.join(os.path.dirname(__file__), 'test_data', 'fake_examples')
 
 
+@dataclasses.dataclass
+class _PathState:
+  """Track the metadata associated with the path."""
+  is_gcs: bool
+  is_abs: bool
+
+
 class MockFs(object):
-  """This util wraps mock for the `tf.io.gfile` API.
+  """This util wraps mock for the `tf.io.gfile` / `epath.Path` API.
+
+  This allow to test code which uses absolute paths / GCS path while keeping
+  tests hermetic.
 
   Usage:
 
   ```
-  fs = MockFs()
-  with fs.mock():
+  with MockFs() as fs:
+    # GCS example
+    fs.add_file('gs://bucket/dir/file.txt')
 
-    fs.add_file('/path/to/file1', 'Content of file 1')
+    assert tf.io.gfile.glob('gs://bucket/*/file.txt') == [
+        'gs://bucket/dir/file.txt',
+    ]
 
-    assert tf.io.gfile.exists('/path/to/file1')
-    with tf.io.gfile.GFile('/path/to/file2', 'w') as f:
-      f.write('Content of file 2')
-    tf.io.gfile.rename('/path/to/file1', '/path/to/file1_moved')
-
-    assert fs.files == {
-        '/path/to/file2': 'Content of file 2',
-        '/path/to/file1_moved': 'Content of file 1',
-    }
+    # This also works with absolute paths
+    tf.io.gfile.makedirs('/path/to/')
+    with tf.io.gfile.GFile('/path/to/file.txt', 'w') as f:
+      f.write('Content of file.txt')
   ```
 
-  Attributes:
-    files: Dict[str, str], mapping existing files -> file content
+  Internally, this is done by converting absolute path into local tmp paths:
+
+  * `/absolute/path` -> `/tmp/mocked_file_system/absolute/path`
+  * `gs://path` -> `/tmp/mocked_file_system/gs/path`
+
   """
 
   def __init__(self):
-    self.files = {}
     self._cm = None
+    self._tmp_dir = None
 
   def __enter__(self):
     self._cm = self.contextmanager()
@@ -103,79 +118,183 @@ class MockFs(object):
 
   @contextlib.contextmanager
   def contextmanager(self) -> Iterator['MockFs']:
-    """Open the file."""
+    """Activate the mock file system."""
     with self.mock():
       yield self
 
-  def add_file(self, path, content=None) -> None:
-    path = os.fspath(path)
-    content = 'Content of {}'.format(path) if content is None else content
-    self.files[path] = content
-
-  def _list_directory(self, path):
-    path = os.fspath(path)
-    path = path.rstrip(os.path.sep) + os.path.sep  # Make sure path is a `dir/`
-    return list({
-        # Extract `path/<dirname>/...` -> `<dirname>`
-        os.path.relpath(p, path).split(os.path.sep)[0]
-        for p in self.files
-        if p.startswith(path)
-    })
-
   @contextlib.contextmanager
-  def _open(self, path, mode='r'):
-    """Patch `tf.io.gfile.GFile`."""
-    path = os.fspath(path)
-    if mode.startswith('w'):
-      self.add_file(path, '')
-    is_binary = 'b' in mode
-
-    if path not in self.files:
-      raise FileNotFoundError(f'File {path} does not exists.')
-    content = self.files[path]
-    if is_binary:
-      fobj = io.BytesIO(content.encode('utf-8'))
-    else:
-      fobj = io.StringIO(content)
-
-    with fobj as f:
-      yield f
-      new_content = f.getvalue()  # Update the content
-
-    self.files[path] = new_content.decode('utf-8') if is_binary else new_content  # pytype: disable=attribute-error
-
-  def _rename(self, from_, to, overwrite=False):
-    from_ = os.fspath(from_)
-    to = os.fspath(to)
-    if not overwrite and to in self.files:
-      raise FileExistsError('Cannot overwrite: {} -> {}'.format(from_, to))  # pytype: disable=name-error
-    if from_ not in self.files:
-      raise FileNotFoundError('Cannot rename unknown file: {}'.format(from_))  # pytype: disable=name-error
-    self.files[to] = self.files.pop(from_)
-
-  def _exists(self, path: str) -> bool:
-    """Returns True, if any file/directory exists."""
-    path = os.fspath(path)
-    path = path.rstrip(os.path.sep)  # Normalize path
-    # Check full path existence
-    if path in self.files:
-      return True
-    # Check parent directory
-    path = path + os.path.sep
-    if any(f.startswith(path) for f in self.files):
-      return True
-    return False
-
   def mock(self):
-    return mock_tf(
-        'tf.io.gfile',
-        exists=self._exists,
-        makedirs=lambda _: None,
-        # Used to get name of file as downloaded:
-        listdir=self._list_directory,
-        GFile=self._open,
-        rename=self._rename,
+    with tempfile.TemporaryDirectory() as tmp_dir_:
+      assert not self._tmp_dir
+      self._tmp_dir = pathlib.Path(tmp_dir_)
+      with self._mock() as m:
+        yield m
+      self._tmp_dir = None
+      # TODO(epot): recursivelly record all
+
+  def _to_tmp(self, p, *, with_state: bool = False):
+    """Normalize the path by returning `tmp_path / p`."""
+    # If `p` was a `epath.Path`, it doesn't matter the value of `is_gcs`
+    # as returned values will be normalized anyway.
+    p_str = os.fspath(p)
+    state = _PathState(
+        is_gcs=p_str.startswith('gs://'),
+        is_abs=p_str.startswith('/'),
     )
+    if state.is_gcs:
+      p = os.fspath(p).replace('gs://', '/big' + 'store/', 1)
+    p = pathlib.Path(p)
+    if p.anchor:
+      p = pathlib.Path(*p.parts[1:])  # Strip leading `/`
+    assert not p.is_absolute()
+    out_p = self._tmp_dir / p
+
+    # Propagate `is_gcs`, so results are consistents:
+    # * tf.io.gfile.glob('gs://')
+    # * tf.io.gfile.glob('/big' + 'store/')
+    if with_state:
+      return out_p, state
+    else:
+      return out_p
+
+  def _to_abs(self, p, *, state: _PathState):
+    """Normalize the output to strip the `tmp_path`."""
+    tmp_path = os.fspath(self._tmp_dir)
+    assert p.startswith(tmp_path)
+    p = p[len(tmp_path):]  # Strip the tmp path
+    if state.is_gcs:
+      assert p.startswith('/big' + 'store/')
+      p = p.replace('/big' + 'store/', 'gs://', 1)
+    elif not state.is_abs:
+      assert p.startswith('/')
+      p = p[len('/'):]
+    return p
+
+  def _validate_out(self, out):
+    """Sanity check to avoid leaking accidentally the `self.tmp_dir`."""
+    if isinstance(out, list):
+      assert not any(elem.startswith('/') for elem in out)
+    elif not isinstance(out, (bool, type(None))):
+      raise TypeError(
+          f'Unexpected return type {out!r} for MockFs, please open an issue')
+    return out
+
+  def add_file(self, path, content=None) -> None:
+    """Add a file, creating all parent directories."""
+    path = os.fspath(path)
+    content = f'Content of {path}' if content is None else content
+    fpath = self._to_tmp(path)
+    fpath.parent.mkdir(parents=True, exist_ok=True)  # pytype: disable=attribute-error
+    fpath.write_text(content)  # pytype: disable=attribute-error
+
+  def read_file(self, path) -> str:
+    return self._to_tmp(path).read_text()  # pytype: disable=attribute-error
+
+  def _mock_open(self, original_fn, p, mode='r', **kwargs):
+    return original_fn(self._to_tmp(p), mode, **kwargs)
+
+  def _mock_fn(self, original_fn, p, **kwargs):
+    return self._validate_out(original_fn(self._to_tmp(p), **kwargs))
+
+  def _mock_fn_2_args(self, original_fn, p, p2, **kwargs):
+    return self._validate_out(
+        original_fn(
+            self._to_tmp(p),
+            self._to_tmp(p2),
+            **kwargs,
+        ))
+
+  def _mock_glob(self, original_fn, p):
+    p, state = self._to_tmp(p, with_state=True)
+    p_outs = original_fn(os.fspath(p))  # tf.io.glob does not accept `pathlib`
+    return [self._to_abs(p_out, state=state) for p_out in p_outs]
+
+  def _mock_walk(self, original_fn, p):
+    p, state = self._to_tmp(p, with_state=True)
+    for (root, subdirs, filenames) in original_fn(p):
+      yield (self._to_abs(root, state=state), subdirs, filenames)
+
+  def _mock(self):
+
+    return mock_gfile(
+        exists=self._mock_fn,
+        listdir=self._mock_fn,
+        isdir=self._mock_fn,
+        remove=self._mock_fn,
+        rmtree=self._mock_fn,
+        mkdir=self._mock_fn,
+        makedirs=self._mock_fn,
+        open=self._mock_open,
+        rename=self._mock_fn_2_args,
+        replace=self._mock_fn_2_args,
+        copy=self._mock_fn_2_args,
+        glob=self._mock_glob,
+        walk=self._mock_walk,
+    )
+
+  def print_tree(self) -> None:
+    print(_get_folder_str(self._tmp_dir))
+
+
+def _get_folder_str(root_dir: pathlib.Path) -> str:
+  """Get the tree structure."""
+  lines = epy.Lines()
+  for p in root_dir.iterdir():
+    if p.is_dir():
+      lines += f'{p.name}/'
+      with lines.indent():
+        subfolder_str = _get_folder_str(p)
+        if subfolder_str:
+          lines += subfolder_str
+    else:
+      lines += p.name
+  return lines.join()
+
+
+@contextlib.contextmanager
+def mock_gfile(**fns: Any) -> Iterator[None]:
+  """Patch `tf.io.gfile.GFile` and `epath.Path`.
+
+  Example: Validate `exists` usage:
+
+  ```
+  def new_exists(old_exists, path):
+    assert not os.fspath(path).startswith('gs://')
+    return old_exists(path)
+
+  with mock_gfile(exists=new_exists)
+  ```
+
+  Args:
+    **fns: Functions to overwrite. Have signature: `fn(original_fn, *args,
+      **kwargs)`. (note the first function argument which allow to access the
+      original function)
+
+  Yields:
+    None
+  """
+  # Process epath kwargs
+  epath_kwargs = {k: fn for k, fn in fns.items() if k != 'walk'}
+
+  # Process gfile kwargs
+  epath_to_gfile_mapping = {
+      'open': 'GFile',
+  }
+  gfile_kwargs = {}
+  for k, fn in fns.items():
+    if k == 'replace':
+      continue
+    gfile_k = epath_to_gfile_mapping.get(k, k)
+    original_fn = getattr(tf.io.gfile, gfile_k)
+    mocked_fn = functools.wraps(original_fn)(functools.partial(fn, original_fn))
+    gfile_kwargs[gfile_k] = mocked_fn
+
+  with contextlib.ExitStack() as stack:
+    cm_epath = epath.testing.mock_epath(**epath_kwargs)
+    cm_gfile = mock_tf('tf.io.gfile', **gfile_kwargs)
+    stack.enter_context(cm_epath)
+    stack.enter_context(cm_gfile)
+    yield
 
 
 @contextlib.contextmanager
@@ -193,23 +312,23 @@ def mock_tf(symbol_name: str, *args: Any, **kwargs: Any) -> Iterator[None]:
   Yields:
     None
   """
-  # pylint: disable=g-import-not-at-top,reimported
-  import tensorflow as tf_lib1
-  import tensorflow as tf_lib2
-  # pylint: enable=g-import-not-at-top,reimported
 
   tf_symbol, *tf_submodules, symbol_name = symbol_name.split('.')
   if tf_symbol != 'tf':
     raise ValueError('Symbol name to patch should start by `tf`.')
 
   with contextlib.ExitStack() as stack:
-    # Patch both `tf` and `tf.compat.v2`
-    for tf_lib in (tf_lib1, tf_lib2):
-      # Recursivelly load the submodules/subobjects (e.g. `tf.io.gfile`)
-      module = tf_lib
-      for submodule in tf_submodules:
-        module = getattr(module, submodule)
-      getattr(module, symbol_name)  # Trigger the lazy-loading of the TF API.
+    # Recursivelly load the submodules/subobjects (e.g. `tf.io.gfile`)
+    module = tf
+    for submodule in tf_submodules:
+      module = getattr(module, submodule)
+    getattr(module, symbol_name)  # Trigger the lazy-loading of the TF API.
+    if kwargs:  # Patch each attribute individually
+      assert not args
+      for k, v in kwargs.items():
+        stack.enter_context(
+            mock.patch.object(getattr(module, symbol_name), k, v))
+    else:
       # Patch the module/object
       stack.enter_context(
           mock.patch.object(module, symbol_name, *args, **kwargs))
